
'''
Prozess-Simulation  ** Realtime -> mqtt
===============================================
(c) 2024 Dr. Jürgen Bechtloff, Meschede
---------------------------------------
+ class-Lib


'''
import simLib_01 as simLib

from pymongo import MongoClient

import numpy as np
from datetime import datetime, timedelta
import time
from benedict import benedict
import json

import os
from apscheduler.schedulers.blocking import BlockingScheduler

# ------------------------------------------------------------------
cfg = benedict.from_toml('cfg.toml', keyattr_dynamic=True)

# ------------------------------------------------------------------

# ------------------------------------------------------------------
class SimProz():

    def __init__(self,bProz) -> None:
        self.PL = {'ts':datetime.now(), 'cnt':0}
        self.bProz = bProz

    def sim(self):
        t0, tG, cntG, y0 = self.bProz.genTS()
        ts = datetime.now()
        print(f'ts= {ts},  cnt={cntG}  y= {y0}')

        self.PL =   {"ts": ts,   # utf-Zeit
                        "cnt": cntG,
                    }
        for i, p in zip(range(self.bProz.m-1), self.bProz.xLst):
            self.PL.update({p.name: y0[i]})

    def init_mdb(self,ip,port,db,co):
        client = MongoClient(ip, port)
        mdb = client[db]
        self.col = mdb[co]
        self.col.drop()  # Alte Collection LÖSCHEN !!!

        try:
            mdb.create_collection(
                co,
                timeseries={
                    "timeField": "ts",
                    "metaField": "loc",
                    "granularity": "seconds"
                },
                # expireAfterSeconds = 4 * 7 * 24 * 60 * 60 # Ablauf der Gültigkeit von Datensätzen (damit die DB nicht unendlich wächst..)  -> CFG
                expireAfterSeconds = 10 * 60 # Ablauf der Gültigkeit von Datensätzen (damit die DB nicht unendlich wächst..)  -> CFG
            )
        # except mdbErrors.OperationFailure:  # If the collection doesn't exist
        except:  # If the collection doesn't exist
            print("This collection doesn't exist")        

    def write_mdb(self, ts):
        self.sim()
        self.PL["ts"] = ts
        result = self.col.insert_one(self.PL)

    def write_mdb_now(self):
        self.sim()
        result = self.col.insert_one(self.PL)
    
# ------------------------------------------------------------------
# ------------------------------------------------------------------


cfg_sim = benedict.from_toml('prz_01.toml', keyattr_dynamic=True)
# ------------------------------------------------------------------

# ------------------------------------------------------------------

bProz = simLib.BatchProz(cfg_sim, cfg_sim.T)
bProz.prep()

simProz = SimProz(bProz)

simProz.init_mdb("127.0.0.1",27017,"rawdata","mmm")

# ------------------------------------------------------------------
tsE = datetime.now()  # now
tsA = tsE - timedelta(hours=1)
# tsDeltaAE = tsE - tsA

ts = tsA
while ts <= tsE: 
    simProz.write_mdb(ts)
    ts = ts + timedelta(seconds=cfg_sim.T)
    print(ts)



# ------------------------------------------------------------------

sched = BlockingScheduler({'apscheduler.timezone': 'UTC'})
# sched.add_job(simProz.sim, 'interval', seconds=cfg_sim.T, id='run_sim')
sched.add_job(simProz.write_mdb_now, 'interval', seconds=cfg_sim.T, id='run_mdbwrite')

print('Press Ctrl+{0} to exit'.format('Break' if os.name == 'nt' else 'C'))

try:
    sched.start()
except (KeyboardInterrupt, SystemExit):
    pass                
